﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Office.Interop.UccApi;
using System.Windows.Forms;

namespace UCCApiConferenceSample.UCCClasses
{
  public class MyConferenceSession : _IUccConferenceSessionEvents, _IUccConferenceSessionParticipantEvents,
    _IUccInstantMessagingSessionParticipantEvents, _IUccSessionEvents, _IUccConferenceManagerSessionEvents, _IUccApplicationSessionParticipantEvents
  {
    private MyEndpoint endpoint;
    public string conferenceUri;
    private IUccConferenceSession confSession;
    public IUccInstantMessagingSession imSession;
    

    public void CreateConferenceSession(MyEndpoint endpoint, string conferenceUri)
    {
      this.endpoint = endpoint;
      this.conferenceUri = conferenceUri;

      IUccSessionManager sessionManager = (IUccSessionManager)endpoint.uccEndpoint;

      IUccSession session = sessionManager.CreateSession(UCC_SESSION_TYPE.UCCST_CONFERENCE, null);
      confSession = (IUccConferenceSession)session;

      ComHelper.Advise<_IUccConferenceSessionEvents>(confSession, this);

      confSession.Enter(UccHelper.GetUri(conferenceUri), null);

    }

    public void OnEnter(UccConferenceSession pEventSource, IUccOperationProgressEvent pEventData)
    {
      if (pEventData.StatusCode != 0)
        throw new Exception("Nope");
            

      IUccConferenceSessionParticipant localParticipant = (IUccConferenceSessionParticipant)((IUccSession)pEventSource).LocalParticipant;
      UccConferenceSessionParticipantEndpoint partEndpoint = localParticipant.CreateParticipantEndpoint(this.endpoint.uccEndpoint.Uri, this.endpoint.uccEndpoint.Id, UCC_CONFERENCE_ENTITY_TYPE.UCCCET_INSTANT_MESSAGING, null);
      ComHelper.Advise<_IUccConferenceSessionParticipantEvents>(localParticipant, this);

      IUccMediaChannel channel = partEndpoint.CreateChannel(UCC_MEDIA_TYPES.UCCMT_MESSAGE, null);
      ((IUccMediaChannel)channel).PreferredMedia = (int)(UCC_MEDIA_DIRECTIONS.UCCMD_RECEIVE | UCC_MEDIA_DIRECTIONS.UCCMD_SEND);
      partEndpoint.AddChannel(channel);

      ((IUccSession)pEventSource).LocalParticipant.AddParticipantEndpoint((IUccSessionParticipantEndpoint)partEndpoint, null);
    }

    public void OnLeave(UccConferenceSession pEventSource, IUccOperationProgressEvent pEventData)
    {
      
    }

    public void OnPropertyUpdated(UccConferenceSession pEventSource, UccPropertyUpdateEvent pEventData)
    {
      
    }

    public void OnSetProperty(UccConferenceSession pEventSource, IUccOperationProgressEvent pEventData)
    {
      
    }


    public void OnAddEndpoint(UccConferenceSessionParticipant pEventSource, IUccOperationProgressEvent pEventData)
    {
      if (pEventData.StatusCode != 0)
        throw new Exception("Nope");

      IUccSessionManager sessionManager = (IUccSessionManager)this.endpoint.uccEndpoint;
      IUccSession session = sessionManager.CreateSession(UCC_SESSION_TYPE.UCCST_INSTANT_MESSAGING, null);

      string imMCUUri = confSession.Properties.get_Property((int)UCC_CONFERENCE_SESSION_PROPERTY.UCCCSPID_CONFERENCE_INSTANT_MESSAGING_URI).StringValue;

      IUccInstantMessagingSessionParticipant imParticipant = (IUccInstantMessagingSessionParticipant)session.CreateParticipant(
        UccHelper.GetUri(imMCUUri), null);

      ComHelper.Advise<_IUccInstantMessagingSessionParticipantEvents>(imParticipant, this);

      session.AddParticipant((IUccSessionParticipant)imParticipant);

      this.imSession = (IUccInstantMessagingSession)session;

      ComHelper.Advise<_IUccSessionEvents>(session, this);
    }

    public void OnPropertyUpdated(UccConferenceSessionParticipant pEventSource, UccPropertyUpdateEvent pEventeData)
    {
     
    }

    public void OnRemoveEndpoint(UccConferenceSessionParticipant pEventSource, IUccOperationProgressEvent pEventData)
    {
     
    }

    public void OnSetProperty(UccConferenceSessionParticipant pEventSource, IUccOperationProgressEvent pEventData)
    {
     
    }

    public void OnComposing(UccInstantMessagingSessionParticipant pEventSource, UccInstantMessagingComposingEvent pEventData)
    {
      
    }

    public void OnIdle(UccInstantMessagingSessionParticipant pEventSource, UccInstantMessagingComposingEvent pEventData)
    {
      
    }

    public void OnInstantMessageReceived(UccInstantMessagingSessionParticipant pEventSource, UccIncomingInstantMessageEvent pEventData)
    {
      MessageBox.Show(pEventData.Content);
    }

    public void OnAddParticipant(IUccSession pEventSource, IUccOperationProgressEvent pEventData)
    {
      if (pEventData.IsComplete == false)
        return;

      if (pEventData.StatusCode != 0)
        throw new Exception("Nope");
    }

    public void OnRemoveParticipant(IUccSession pEventSource, IUccOperationProgressEvent pEventData)
    {

    }

    public void OnTerminate(IUccSession pEventSource, IUccOperationProgressEvent pEventData)
    {

    }

    public void ScheduleConference(MyEndpoint endpoint, string strFocusUri)
    {
      UccUriManager uriManager = new UccUriManager();
      UccUri focusUri = uriManager.ParseUri(strFocusUri);

      IUccConferenceManager confManager = (IUccConferenceManager)endpoint.uccEndpoint;
      IUccConferenceManagerSession confManagerSession = confManager.CreateConferenceManagerSession(focusUri);

      ComHelper.Advise<_IUccConferenceManagerSessionEvents>(confManagerSession, this);

      UccConferenceInformation ci = confManagerSession.CreateConferenceInformation();
      ci.Properties.AddProperty((int)UCC_CONFERENCE_INFORMATION_PROPERTY.UCCCIP_ADMISSION_TYPE, UCC_CONFERENCE_ADMISSION_TYPE.UCCCAT_OPEN_AUTHENTICATED);

      UccPropertyCollection mcuAttributeCollection = new UccPropertyCollection();
      mcuAttributeCollection.AddProperty((int)UCC_CONFERENCE_ENTITY_SETTING_PROPERTY.UCCCESP_TYPE, UCC_CONFERENCE_ENTITY_TYPE.UCCCET_INSTANT_MESSAGING);

      UccPropertyCollection mcuPropertyCollection = new UccPropertyCollection();
      mcuPropertyCollection.AddProperty((int)UCC_CONFERENCE_ENTITY_TYPE.UCCCET_INSTANT_MESSAGING, mcuAttributeCollection);

      ci.Properties.AddProperty((int)UCC_CONFERENCE_INFORMATION_PROPERTY.UCCCIP_MCUS, mcuPropertyCollection);

      UccOperationContext operationContext = new UccOperationContext();
      UccContext mcuContext = new UccContext();
      mcuContext.AddNamedProperty("ScheduleConference", true);
      operationContext.Initialize(1, mcuContext);

      confManagerSession.ScheduleConference(ci, operationContext);
    }

    public void OnDeleteConference(UccConferenceManagerSession pEventSource, IUccOperationProgressEvent pEventData)
    {
      
    }

    public void OnGetAvailableMcuList(UccConferenceManagerSession pEventSource, IUccOperationProgressEvent pEventData)
    {
      
    }

    public void OnGetConference(UccConferenceManagerSession pEventSource, IUccOperationProgressEvent pEventData)
    {
      
    }

    public void OnGetConferenceList(UccConferenceManagerSession pEventSource, IUccOperationProgressEvent pEventData)
    {
      
    }

    public void OnModifyConference(UccConferenceManagerSession pEventSource, IUccOperationProgressEvent pEventData)
    {
      
    }

    public void OnScheduleConference(UccConferenceManagerSession pEventSource, IUccOperationProgressEvent pEventData)
    {
      if (pEventData.StatusCode != 0)
        throw new Exception("Error scheduling conference");

      UccProperty confUriProperty = pEventData.Properties.get_Property((int)UCC_CONFERENCE_MANAGER_OPERATION_COMPLETED_EVENT_PROPERTY.UCCCMOCEP_CONFERENCE_URI);
      conferenceUri = confUriProperty.StringValue;
    }

    public void InviteUser(IUccEndpoint endpoint, string userUri, string conferenceUri)
    {
      this.conferenceUri = conferenceUri;

      UccContext inviteContext = new UccContext();
      inviteContext.AddNamedProperty("ConfInvitee", userUri);

      IUccSessionManager sessionManager = (IUccSessionManager)endpoint;
      IUccSession session = sessionManager.CreateSession(UCC_SESSION_TYPE.UCCST_APPLICATION, inviteContext);

      UccUriManager uriManager = new UccUriManager();
      UccUri userUriUri = uriManager.ParseUri(userUri);

      IUccSessionParticipant participant = session.CreateParticipant(userUriUri, inviteContext);
      ComHelper.Advise<_IUccApplicationSessionParticipantEvents>(participant, this);
      session.AddParticipant(participant, null);
    }



    public void OnIncomingInvitation(UccApplicationSessionParticipant pEventSource, UccIncomingInvitationEvent pEventData)
    {
      
    }

    public void OnInvitationAccepted(UccApplicationSessionParticipant pEventSource, UccInvitationAcceptedEvent pEventData)
    {
      
    }

    public void OnOutgoingInvitation(UccApplicationSessionParticipant pEventSource, UccOutgoingInvitationEvent pEventData)
    {
      pEventData.ContentType = "application/ms-conf-invite+xml";
      pEventData.SessionDescription =
        "<Conferencing version=\"2.0\"><focus-uri>" + this.conferenceUri + "</focus-uri><subject>Conference</subject><im available=\"true\"><first-im></first-im></im></Conferencing>";

      pEventData.Send();
    }

    public void OnRenegotiate(UccApplicationSessionParticipant pEventSource, IUccOperationProgressEvent pEventData)
    {
      
    }
  }
}
