﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Office.Interop.UccApi;
using System.Windows.Forms;
using System.IO;
using System.Xml;

namespace UCCApiConferenceSample.UCCClasses
{
  public class MyEndpoint : _IUccEndpointEvents, _IUccSessionDescriptionEvaluator, _IUccSessionManagerEvents
  {        
    public IUccEndpoint uccEndpoint;
    public MyConferenceSession confSession;
    public Form1 form1;

    public void EnableEndpoint(MyPlatform platform, string UserUri, string UserName, string UserPassword, string UserDomain, string ServerName, UCC_TRANSPORT_MODE transportMode)
    {
      UccUri uri = UccHelper.GetUri(UserUri);
      uccEndpoint = platform.uccPlatform.CreateEndpoint(UCC_ENDPOINT_TYPE.UCCET_PRINCIPAL_SERVER_BASED, uri, null, null);

      ComHelper.Advise<_IUccEndpointEvents>(uccEndpoint, this);     

      IUccSessionManager sm = (IUccSessionManager)uccEndpoint;
      sm.RegisterSessionDescriptionEvaluator(this);
      ComHelper.Advise<_IUccSessionManagerEvents>(sm, this);
      

      IUccServerSignalingSettings signalingSettings = (IUccServerSignalingSettings)uccEndpoint;
      
      UccCredential credential = signalingSettings.CredentialCache.CreateCredential(UserName, UserPassword, UserDomain);
      signalingSettings.CredentialCache.SetCredential("*", credential);
      signalingSettings.AllowedAuthenticationModes = (int)UCC_AUTHENTICATION_MODES.UCCAM_NTLM;
      signalingSettings.Server = signalingSettings.CreateSignalingServer(ServerName, transportMode);

      uccEndpoint.Enable(null);
    }


    public void OnDisable(IUccEndpoint pEventSource, IUccOperationProgressEvent pEventData)
    {
      
    }

    public void OnEnable(IUccEndpoint pEventSource, IUccOperationProgressEvent pEventData)
    {
      if (pEventData.StatusCode != 0)
        throw new Exception("Bad luck");
      MessageBox.Show("Endpoint enabled");      
    }

    public bool Evaluate(string bstrContentType, string bstrSessionDescription)
    {
      if (bstrContentType == "application/ms-conf-invite+xml")
        return true;
      else
        return false;
    }

    public void OnIncomingSession(IUccEndpoint pEventSource, UccIncomingSessionEvent pEventData)
    {
      if (pEventData.Session.Type == UCC_SESSION_TYPE.UCCST_APPLICATION)
      {
        pEventData.Accept();
        UccIncomingInvitationEvent inInvitation = (UccIncomingInvitationEvent)pEventData;
        
        TextReader tr = new StringReader(inInvitation.RemoteSessionDescription);
        XmlDocument inviteDoc = new XmlDocument();
        inviteDoc.Load(tr);

        // Get the conference Focus URI from the XML stream.
        string conferenceUri = inviteDoc.SelectSingleNode("Conferencing/focus-uri").InnerText;

        confSession = new MyConferenceSession();
        confSession.CreateConferenceSession(this, conferenceUri);
      }
      else if (pEventData.Session.Type == UCC_SESSION_TYPE.UCCST_INSTANT_MESSAGING)
      {
        pEventData.Accept();
        form1.myIMSession = new MyIMSession();
        form1.myIMSession.ProcessIncomingSession(pEventData);
      }
    }

    public void OnOutgoingSession(IUccEndpoint pEventSource, UccOutgoingSessionEvent pEventData)
    {
      
    }
  }
}
