﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Office.Interop.UccApi;
using System.Windows.Forms;

namespace UCCApiConferenceSample.UCCClasses
{
  public class MySubscriptionManager : _IUccPresentityEvents, _IUccCategoryContextEvents
  {
    private MyEndpoint myEndpoint;
    public UccUri FocusUri;

    public MySubscriptionManager(MyEndpoint endpoint)
    {
      myEndpoint = endpoint;
    }

    public void SubscribeServerConfiguration()
    {
      IUccSubscriptionManager subscriptionManager = (IUccSubscriptionManager)myEndpoint.uccEndpoint;

      IUccSubscription serverConfigurationSubscription = subscriptionManager.CreateSubscription(null);

      UccPresentity presentity = serverConfigurationSubscription.CreatePresentity(myEndpoint.uccEndpoint.Uri, null);
      ComHelper.Advise<_IUccPresentityEvents>(presentity, this);
      serverConfigurationSubscription.AddPresentity(presentity);
      serverConfigurationSubscription.AddCategoryName("ServerConfiguration");
      serverConfigurationSubscription.Query(null);
    }

    public void SubscribeContactCard()
    {
      IUccSubscriptionManager subscriptionManager = (IUccSubscriptionManager)myEndpoint.uccEndpoint;

      IUccSubscription serverConfigurationSubscription = subscriptionManager.CreateSubscription(null);

      UccPresentity presentity = serverConfigurationSubscription.CreatePresentity(myEndpoint.uccEndpoint.Uri, null);
      ComHelper.Advise<_IUccPresentityEvents>(presentity, this);
      serverConfigurationSubscription.AddPresentity(presentity);
      serverConfigurationSubscription.AddCategoryName("contactCard");
      serverConfigurationSubscription.Subscribe(null);
    }

    public void SubscribeState()
    {
      IUccSubscriptionManager subscriptionManager = (IUccSubscriptionManager)myEndpoint.uccEndpoint;

      IUccSubscription serverConfigurationSubscription = subscriptionManager.CreateSubscription(null);

      UccPresentity presentity = serverConfigurationSubscription.CreatePresentity(myEndpoint.uccEndpoint.Uri, null);
      ComHelper.Advise<_IUccPresentityEvents>(presentity, this);
      serverConfigurationSubscription.AddPresentity(presentity);
      serverConfigurationSubscription.AddCategoryName("state");
      serverConfigurationSubscription.Subscribe(null);
    }

    public void SubscribeContacts()
    {
      IUccSubscriptionManager subscriptionManager = (IUccSubscriptionManager)myEndpoint.uccEndpoint;

      IUccSubscription serverConfigurationSubscription = subscriptionManager.CreateSubscription(null);

      UccPresentity presentity = serverConfigurationSubscription.CreatePresentity(myEndpoint.uccEndpoint.Uri, null);
      ComHelper.Advise<_IUccPresentityEvents>(presentity, this);
      serverConfigurationSubscription.AddPresentity(presentity);
      serverConfigurationSubscription.AddCategoryName("contacts");
      serverConfigurationSubscription.Subscribe(null);
    }

    public void SubscribeContainers()
    {
      IUccSubscriptionManager subscriptionManager = (IUccSubscriptionManager)myEndpoint.uccEndpoint;

      IUccSubscription serverConfigurationSubscription = subscriptionManager.CreateSubscription(null);

      UccPresentity presentity = serverConfigurationSubscription.CreatePresentity(myEndpoint.uccEndpoint.Uri, null);
      ComHelper.Advise<_IUccPresentityEvents>(presentity, this);
      serverConfigurationSubscription.AddPresentity(presentity);
      serverConfigurationSubscription.AddCategoryName("containers");
      serverConfigurationSubscription.Subscribe(null);
    }

    public void OnCategoryContextAdded(UccPresentity pPresentity, UccCategoryContextEvent pCategoryCtxtEvent)
    {
      ComHelper.Advise<_IUccCategoryContextEvents>(pCategoryCtxtEvent.CategoryContext, this);
    }

    public void OnCategoryContextRemoved(UccPresentity pPresentity, UccCategoryContextEvent pCategoryCtxtEvent)
    {
      
    }

    public void OnCategoryInstanceAdded(IUccCategoryContext pCategoryCtxt, UccCategoryInstanceEvent pCategoryInstanceEvent)
    {
      if (pCategoryCtxt.Name == "ServerConfiguration")
      {
        IUccServerConfigurationCategory category = (IUccServerConfigurationCategory)pCategoryInstanceEvent.CategoryInstance;
        this.FocusUri = category.FocusFactory;        
      }
      else if (pCategoryCtxt.Name == "contactCard")
      {
        IUccPresenceContactCardInstance contactCard = (IUccPresenceContactCardInstance)pCategoryInstanceEvent.CategoryInstance;        
      }
      else if (pCategoryCtxt.Name == "state")
      {
        MyCategoryInstance inst = new MyCategoryInstance();
        ComHelper.Advise<_IUccCategoryInstanceEvents>(pCategoryInstanceEvent.CategoryInstance, inst);

        IUccPresenceStateInstance state = (IUccPresenceStateInstance)pCategoryInstanceEvent.CategoryInstance;
        MessageBox.Show(state.Availability.ToString());        
      }
      else if (pCategoryCtxt.Name == "containers")
      {
        IUccContainer container = (IUccContainer)pCategoryInstanceEvent.CategoryInstance;

        string containerData = "instanceId: " +  pCategoryInstanceEvent.CategoryInstance.InstanceId.ToString() + "\n" +
          "containerId: " + container.Id.ToString() + "\n";

        foreach(IUccContainerMember member in container.Members){
          if (member.Scope == UCC_CONTAINER_MEMBERSHIP_SCOPE.UCCCMS_USER)
            containerData += "memberUri: " + member.Uri + "/Scope: " + member.Scope.ToString() + "\n";          
          else
            containerData += "memberScope: " + member.Scope.ToString() + "\n";          
        }

        MessageBox.Show(containerData);
      }
      else if (pCategoryCtxt.Name == "contacts")
      {
        IUccContact contact = (IUccContact)pCategoryInstanceEvent.CategoryInstance;
        MessageBox.Show("Contact: " + contact.Uri.Value + " / " + contact.Name);
        MyContact myContact = new MyContact();
        myContact.HandleContactEvents(contact, this.myEndpoint);        
      }
    }

    public void OnCategoryInstanceRemoved(IUccCategoryContext pCategoryCtxt, UccCategoryInstanceEvent pCategoryInstanceEvent)
    {
      
    }
  }
}
